/* Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Updated: Fri Mar  8 02:24:23 2002 by Jeff Dalton
 * Copyright: (c) 2000, AIAI, University of Edinburgh
 */

package ix.iface.domain;

import java.io.*;
import java.util.*;

import ix.icore.domain.*;
import ix.util.*;
import ix.util.xml.*;

// Imports for using Xerces parser
// import org.apache.xerces.parsers.DOMParser;
// import org.w3c.dom.Document;
// import org.xml.sax.SAXException;
// import java.io.IOException;

// Imports for using JDOM
// import java.io.IOException;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;


/**
 * A parser for domains described in a simple XML syntax.
 */
public class XMLTF_Parser extends DomainParser {

    String filename;

    public XMLTF_Parser(String filename) throws FileNotFoundException {
	this.filename = filename;
    }

    public XMLTF_Parser(File file) throws FileNotFoundException {
	this(file.getPath());
    }

    public Domain readDomain() {
	return readDomain(new Domain());
    }

    public Domain readDomain(Domain domain) {
	return readDomainJDOM(domain);
    }

    //
    // Parsing with JDOM
    //

    protected String SAXDriverClass = "org.apache.xerces.parsers.SAXParser";

    public Domain readDomainJDOM(Domain domain) {

	try {
	    SAXBuilder builder = new SAXBuilder(SAXDriverClass);
	    Document doc = builder.build(new File(filename));
	    fillInDomain(domain, doc);
	}
	catch (JDOMException jde) {
	    Debug.noteException(jde, false);
	    throw new SyntaxException(jde.toString());
	}
	catch (Exception e) {
	    Debug.noteException(e, false);
	    throw new SyntaxException(e.toString());
	}
	return domain;
    }

    protected void fillInDomain(Domain domain, Document doc) {
	Domain newDomain = (Domain)XML.objectFromDocument(doc);
	domain.takeFrom(newDomain);
    }

}

