/****************************************************************************
 * A frame with a menu bar to close it, and a scrollable text 
 *
 * @author Jussi Stader
 * @version 1.0
 * Copyright: (c) 2001, AIAI, University of Edinburgh
 *
 *****************************************************************************
 */
package ix.iface.ui;

import ix.*;
import ix.util.*;
import ix.iface.ui.util.*;

import java.util.*;
import java.awt.*;
import java.net.URL;
import java.io.IOException;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

/**
 * A frame with a menu bar to close it, and a scrollable text 
 */
public class HelpFrame extends JFrame 
  implements ActionListener, HyperlinkListener 
{
  
  private JEditorPane htmlPane = new JEditorPane();
  private URL original;

  /**
   * Puts up a frame with a menu bar and a scrollable text area that can
   * display HTML (including images), follow links when clicked, and select
   * text, but NOT edit.
   */
  private HelpFrame(){
    super();
    getContentPane().setLayout(new BorderLayout(0,0));
    setSize(550,650);

    JMenuBar menuBar = new JMenuBar();
    setJMenuBar(menuBar);
    JMenu fileMenu = new JMenu("File");	 
    menuBar.add(fileMenu);
    JMenuItem item = new JMenuItem("Close");	 
    item.setActionCommand("close");
    item.addActionListener(this);
    JMenuItem item2 = new JMenuItem("Back to original help page");	 
    item2.setActionCommand("back");
    item2.addActionListener(this);
    fileMenu.add(item2);
    fileMenu.addSeparator();
    fileMenu.add(item);

    /*
    JScrollPane scrollPane = new JScrollPane();
    getContentPane().add(BorderLayout.CENTER, scrollPane);
    scrollPane.setBounds(0,0,400,300);
    mainPanel = new JPanel();
    scrollPane.getViewport().add(mainPanel); 
    mainPanel.add(htmlPane);
    */

    htmlPane.setEditable(false);
    htmlPane.addHyperlinkListener(this);
    JScrollPane htmlView = new JScrollPane(htmlPane);
    getContentPane().add(BorderLayout.CENTER, htmlView);

    setTitle("Help");
  }

  /**
   * As above but the given URL is displayed and remembered as the original
   * page.
   */
  public HelpFrame(URL url){
    this();	
    displayURL(url);
    original = url;
  }
  /**
   * As above but the given file name is converted to a URL using the html
   * area of the resources. 
   * An example call is <code> HelpFrame("ide-help.html") <\code>
   */
  public HelpFrame(String fileName){
    this();	
    try {
      URL url = UIUtil.resourceURL(fileName);
      original = url;
      displayURL(url);
    }
    catch (Exception e) {
      Debug.noteException(e);
    }
  }
  	

  /**
   * Displays the given URL in the text area, but does not remember it as the 
   * original page.
   */
  public boolean displayURL(URL url) {
    Debug.noteln("Displaying url", url.toString());
    try {
      htmlPane.setPage(url);
      return true;
    } 
    catch (IOException e) {
      if ((url != null) && (!url.getProtocol().equals("http"))) 
	UIUtil.warning(this, "Cannot deal with non-http links in URL: " + 
		       url.toString());
      else
	UIUtil.warning(this, "Attempted to read a bad URL: " + url.toString());
      return false;
    }
  }
  private void displayOriginal() {
    if (original != null)
      this.displayURL(original);
    else 
      UIUtil.warning(this, "No original help page found");
  }

  /**
   * Gets called when a link is clicked in the current page. Use this to 
   * implement following links.
   * The events seem to be of type "ACTIVATED". Other types are ENTERED and 
   * EXITED but I don't know what they are!
   */
  public void hyperlinkUpdate(HyperlinkEvent e) {
    //Debug.noteln("HelpFrame got hyperlink update", e.getDescription());
    //Debug.noteln(" type", e.getEventType().toString());
    //only do things when clicked (not interested in enter/exit events)
    if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
      URL current = htmlPane.getPage();
      if (!displayURL(e.getURL())) {
	Debug.noteln("The bad link's description is", e.getDescription());
	displayURL(current);
      }
    }
  }

  public void actionPerformed(java.awt.event.ActionEvent event) {
    Object object = event.getSource();
    String command = ((AbstractButton)object).getActionCommand();
    //Debug.noteln("action object is",object);
    Debug.noteln("HelpFrame: action command is",command);
    if (command == "close") this.setVisible(false);
    else if (command == "back") this.displayOriginal();
    else {
      String txt = ((AbstractButton)object).getText();
      if ((txt == "") || txt == null) txt = command;
      UIUtil.notImplemented(this, txt);
    }
  }
}
/*
 * Document all public methods
 */
