/****************************************************************************
 * A TableCellEditor for HandlerActions of AgendaItems.
 *
 * @author Jussi Stader
 * @version 1.0
 * Copyright: (c) 2001, AIAI, University of Edinburgh
 *
 *****************************************************************************
 */
package ix.ip2;

import ix.util.*;
import ix.iface.ui.*;
import ix.iface.ui.table.*;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import java.util.*;


/****************************************************************************
 * A TableCellEditor for HandlerActions of AgendaItems.
 *****************************************************************************
 */
public class HandlerActionEditor extends IXDefaultCellEditor 
{
  List handlerActions;
  HandlerAction currentAction = null;
  AgendaItem item;
  AgendaTableViewer viewer;

  public HandlerActionEditor(AgendaTableViewer viewer, IXComboBox jcb) {
    super(jcb);
    setClickCountToStart(1); 
    jcb.setEditable(false);
    jcb.setRenderer(new NDRenderer(true));
    this.viewer = viewer;
  }

  public Component getTableCellEditorComponent(JTable table, Object value,
					       boolean selected,
					       int row, int col) {
    //Add all items that should go in - remember not to add the right one if 
    // several have the same description. They cannot be distinguished by the
    // user. Use the one that the user chose.
    ArrayList items = new ArrayList(); //the items that really go into the list
    JComboBox jcb = (JComboBox)editorComponent;
    if (jcb.getItemCount() != 0) jcb.removeAllItems();

    AgendaItemTableModel model = (AgendaItemTableModel)table.getModel();
    item = (AgendaItem)model.getRowObject(row);
    //make up a list of things that should really go into the editor
    for (Iterator i = item.getActions().iterator(); i.hasNext(); ) {
      //get the next action in the list
      HandlerAction act = (HandlerAction)i.next();
      //make sure this is the one we are using (user works with descriptions)
      String description = act.getActionDescription();
      act = viewer.findHandlerAction(item, description);
      //do not add it again if it is already there
      if (!items.contains(act)) items.add(act);
    }
    for (Iterator i = items.iterator(); i.hasNext(); ) {
      //get the next action in the list
      HandlerAction act = (HandlerAction)i.next();
      //Set the colour to the same as the item status or grey if not ready
      Color colour;
      if (act.isReady()) colour = item.getStatus().getColor();
      else colour = Color.gray;
      jcb.addItem(new DefaultColourField(act.getActionDescription(), colour));
    }
    return super.getTableCellEditorComponent(table, value, selected, row, col);
  }

}

  
