/* Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Updated: Fri Mar  8 01:02:31 2002 by Jeff Dalton
 * Copyright: (c) 2001 - 2002, AIAI, University of Edinburgh
 */

package ix.ip2;

import javax.swing.*;

import java.awt.Container;
import java.awt.Component;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.*;

import java.util.*;

import ix.ip2.event.*;

import ix.iview.SimpleDomainEditor;

import ix.icore.domain.Refinement;

import ix.icore.process.PNode;

import ix.icore.*;
import ix.util.*;
import ix.util.lisp.*;

/**
 * A simple editor for entering and viewing Issues.
 */
class IssueEditor extends ItemEditor {

    IssueEditor(AgendaViewer agendaViewer, String title) {
	super(agendaViewer, title);
    }

    protected EditPanel makeEditPanel() {
	EditPanel p = new IssueEditPanel();
	p.setBorder(BorderFactory.createTitledBorder("Issue"));
	return p;
    }

    /**
     * Constructs the menu bar
     */
    protected JMenuBar makeMenuBar() {
	JMenuBar bar = new JMenuBar();

	JMenu fileMenu = new JMenu("File");
	bar.add(fileMenu);

	fileMenu.add(makeMenuItem("Close"));

	return bar;
    }

    /**
     * Action interpreter
     */
    public void actionPerformed(ActionEvent e) {
	String command = e.getActionCommand();
	Debug.noteln("IssueEditor action:", command);
	if (command.equals("Close")) {
	    frame.setVisible(false);
	}
	else
	    Debug.noteln("Nothing to do for", command);
    }

    class IssueEditPanel extends EditPanel {

	IssueEditPanel() {
	    super();
	    modifyButton = makeButton("Modify Issue");
	    setUp();
	}

	protected void setUp() {

	    add(makeLeftLabel("Issue Pattern Text"));
	    add(new JScrollPane(itemText));

	    add(makeLeftLabel("Annotations"));
	    add(new JScrollPane(commentText));

	    newItemButtons.add(makeButton("Add Issue"));
	    newItemButtons.add(makeButton("Clear"));
	    newItemButtons.add(makeButton("Cancel"));

	    editItemButtons.add(modifyButton);
	    editItemButtons.add(makeButton("Copy Issue"));
	    editItemButtons.add(makeButton("Clear"));
	    editItemButtons.add(makeButton("Cancel"));

	    add(newItemButtons);
	}

	/**
	 * Action interpreter
	 */
	public void actionPerformed(ActionEvent e) {
	    String command = e.getActionCommand();
	    Debug.noteln("Issue action:", command);
	    if (command.equals("Add Issue")) {
		addItem();
	    }
	    else if (command.equals("Modify Issue")) {
		modifyItem();
	    }
	    else if (command.equals("Copy Issue")) {
		copyItem();
	    }
	    else if (command.equals("Clear")) {
		showNewItem();
	    }
	    else if (command.equals("Cancel")) {
		// Equiv to close + clear.
		frame.setVisible(false);
		showNewItem();
	    }
	    else
		Debug.noteln("Nothing to do for", command);
	}

    }

}

