/* Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Updated: Tue May 14 20:32:57 2002 by Jeff Dalton
 * Copyright: (c) 2001, AIAI, University of Edinburgh
 */

package ix.ispace;
import java.util.*;
import ix.util.*;

/**
 * What we know about another agent.
 */
public class AgentData {

    protected String name;
    protected AgentRelationship relationship;
    protected List capabilities = new LinkedList(); // Set? /\/

    public AgentData() {
    }

    public AgentData(String name, AgentRelationship relationship) {
	this.name = name;
	this.relationship = relationship;
    }

    public AgentData(AgentData oldData) {
	this.name = oldData.name;
	this.relationship = oldData.relationship;
	this.capabilities = new LinkedList(oldData.capabilities);
    }

    public String getName() { return name; }
    public void setName(String name) { this.name = name; }

    // /\/: Make names consistent
    // /\/: Maybe rename AgentRelationship to AgentRelation.
    public AgentRelationship getRelationship() {
	return relationship;
    }
    public void setRelationship(AgentRelationship relationship) {
	this.relationship = relationship;
    }

    public List getCapabilities() {
	return capabilities;
    }

    public void addCapability(Capability c) {
	Debug.noteln("Agent " + name + " gets " + c);
	capabilities.add(c);
    }

    public boolean hasCapability(Capability c) {
	for (Iterator i = capabilities.iterator(); i.hasNext();) {
	    Capability has = (Capability)i.next();
	    if (has.subsumes(c))
		return true;
	}
	return false;
    }

    public String toString() {
	return "AgentData[" + name + ", " + relationship + "]";
    }

}
