/* Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Updated: Tue May 21 00:56:29 2002 by Jeff Dalton
 * Copyright: (c) 2001, AIAI, University of Edinburgh
 */

package ix.ispace.event;

import java.util.EventObject;
import ix.ispace.*;

public class ContactEvent extends EventObject {

    protected AgentData oldData;
    protected AgentData newData;

    public ContactEvent(ContactManager source, 
			AgentData oldData,
			AgentData newData) {
	super(source);
	this.oldData = oldData;
	this.newData = newData;
    }

    public AgentData getOldData() {
	return oldData;
    }

    public AgentData getNewData() {
	return newData;
    }

    public boolean isNewContact() {
	return oldData == null;
    }

    public boolean isDeletion() {
	return newData == null;
    }

    public boolean isRelationshipChange() {
	return oldData != null && newData != null
	    && oldData.getRelationship() != newData.getRelationship();
    }

    public boolean isCapabilityChange() {
	return oldData != null && newData != null
	    && !oldData.getCapabilities().equals(newData.getCapabilities());
    }

    public String toString() {
	return "ContactEvent[" + oldData + " --> " + newData + "]";
    }

}
