/****************************************************************************
 * An interface model to be used to fill an IFormPanel.
 *
 * @author Jussi Stader
 * @version 1.0
 * Copyright: (c) 2001, AIAI, University of Edinburgh
 *
 *****************************************************************************
 */
package ix.iview;

import ix.*;
import ix.util.Debug;
import ix.util.lisp.*;
import ix.iface.ui.*;
import ix.icore.domain.*;
import ix.iview.domain.*;

import java.util.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * An default model to be used to fill an IFormPanel.
 *
 */
public class ActionFormModel extends GenericIFormModel {
  private static String miniC = "miniConstraints";

  public ActionFormModel() {
    super(UIRefinement.class);
  }
  public ActionFormModel(Class objectClass) {
    super(objectClass);
  }
  public ActionFormModel(Object object) {
    super(object);
  }

  protected Class deriveClass(String field) {
    if (field.equals(miniC)) 
      return String.class;
    else return super.deriveClass(field);
  }
  protected String deriveName(String field) {
    if (field.equals(miniC)) 
      return "Orderings";
    else return super.deriveName(field);
  }


  protected List deriveDisplayFields() {
    //get the class to work with
    List fields = super.deriveDisplayFields();
    fields.add(miniC);
    return fields;
  }

  public Object getValue(String field) {
    if (baseObject == null) {
      Debug.noteln("Cannot get value for a null base object.");
      return null;
    }
    else {
      if (field.equals(miniC)) {
	UIRefinement uir =  (UIRefinement)baseObject;
	return uir.getOrderingType();
      }
      else return super.getValue(field);
    }
  }

  public double getFieldWeight(String field) {
    if (field.equals(miniC) || field.equals("pattern")) return 0.1;
    else return super.getFieldWeight(field);
  }

  /**
   * setValue(String field, Object value) calls this too, so only implement one
   * The miniC settings are dealt with as they happen 
   */
  public void setValue(Object o, String field, Object value) {
    if (field.equals(miniC)) return;
    else if (field.equals("nodes")) {
      try { ((UIRefinement)o).setNodes((List)value); }
      catch (Exception e) {
	Debug.noteException(e);
	super.setValue(o, field, value);
      }
    }
    else if (field.equals("issues")) {
      try { ((UIRefinement)o).setIssues((List)value); }
      catch (Exception e) {
	Debug.noteException(e);
	super.setValue(o, field, value);
      }
    }
    else super.setValue(o, field, value);
  }

}
