/****************************************************************************
 * A panel containing form-style sub-panels for viewing/editing an object
 *
 * @author Jussi Stader
 * @version 2.0
 * Copyright: (c) 2001, AIAI, University of Edinburgh
 *
 *****************************************************************************
 */
package ix.iview;

import ix.*;
import ix.util.Debug;
import ix.util.lisp.*;
import ix.iface.ui.*;
import ix.iface.ui.util.*;
import ix.iface.ui.table.*;
import ix.iview.table.*;
import ix.iview.util.*;
import ix.iview.domain.*;
import ix.iview.domain.event.*;
import ix.icore.*;
import ix.icore.domain.*;
import ix.icore.domain.event.*;

import java.util.*;
import java.lang.reflect.*;
//import java.awt.*;
import java.awt.Dimension;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;

/**
 * A panel containing form-style sub-panels for viewing/editing an object
 *
 */
public class GlobalFormPanel extends IFormPanel 
implements ConstructEditing, UIDomainListener {

  private UIObject uiConstruct;

  private ActionTreeTableModel tableModel;
  private AConstructFramePanel parent;

  public GlobalFormPanel(AConstructFramePanel theParent) {
    super();
    parent = theParent;
  }
  public GlobalFormPanel(AConstructFramePanel theParent, IFormModel m) {
    super(m);
    parent = theParent;
  }

  //----------------------------Window buidling things-----------------------

  protected void setRenderers() {
    IXRenderer r = new ActionListRenderer(false);
    //fieldRenderers.put("products", r);
    fieldRenderers.put("refinements", r);
    //r = new DescriptionsRenderer(false);
    //fieldRenderers.put("annotations", r);
  }

  protected IXEditorPanel makeDisplayBit(String field) {
    //Debug.noteln("GlobalFormPanel: making display bit for " + field);
    if (field.equals("refinements")) {
      //Debug.noteln(" making refinements table");
      String[] buttons = {"Add","Edit","Delete"};
      tableModel = new ActionTreeTableModel(getUIDomain());
      JTable table = new IXTreeTable(tableModel);
      table.setPreferredScrollableViewportSize(new Dimension(200, 100));
      JScrollPane jsp = new JScrollPane();
      jsp.getViewport().add(table);
      MouseListener ml = ((GlobalFramePanel)parent).getML();
      return new ListEditorPanel(ml,model.getFieldName(field),jsp,buttons);
    }
    else return super.makeDisplayBit(field);
  }

  //----------------------------Field things--------------------------------

  public void saveFieldData(String field) {
    if (field.equals("annotations"))
      Debug.noteln("Cannot save annotations yet");
    else 
      //the refinements field cannot be updated from the global domain editor
      if (!field.equals("refinements")) super.saveFieldData(field);
      
  }
  public Object getFieldData(String field) {
    //the refinements field cannot be updated from the global domain editor
    if (field.equals("refinements")) return null;
    else return super.getFieldData(field);
  }

  public void setFieldData(String field, Object value) {
    if (field.equals("refinements")) tableModel.reloadData();
    else super.setFieldData(field, value);
    
  }


  //----------------------------UIDomainListening things--------------------

  public void refinementAdded(RefinementEvent e) {
    //tableModel.addData(e.getRefinement());
    //if (!getUIDomain().loading) updateTable();
  }
  public void refinementRemoved(RefinementEvent e) {
    //tableModel.removeData(e.getRefinement());
    //updateTable();
    //tableModel.setData(getUIDomain().getAllRefinements());
  }
  public void constructAdded(RefinementEvent e) {}
  public void constructAdded(UIDomainEvent e) {
  //tableModel.addData(e.getObject());
    //if (!getUIDomain().loading) updateTable();
  }
  public void constructRemoved(UIDomainEvent e) {
    //tableModel.removeData(e.getObject());
    //updateTable();
  }
  public void constructEdited(UIDomainEvent e) {
    //tableModel.removeData(e.getObject());
    //tableModel.addData(e.getObject());
    //updateTable();
  }
  public void domainCleared(UIDomainEvent e) {
    //tableModel.clearData();
  }
  public void domainEdited(UIDomainEvent e) {
    //Debug.noteln("GFoP: domain edited", e.getDomain().getName());
    getModel().setObject(e.getDomain());
    //updateTable();
  }
  public void domainSet(UIDomainEvent e) {
    getModel().setObject(e.getDomain());
    //updateTable();
  }

  public void updateTable() {
    tableModel.reloadData();
  }

  //----------------------------Construct Editing things-----------------------

  public UIDomain getUIDomain() {
    return parent.getUIDomain();
  }
  public void setConstruct(IXObject construct) {
    setUIConstruct(getUIDomain().getUIObject(construct));
  }
  public IXObject getConstruct() {
    return uiConstruct.getBaseObject();
  }
  public void setUIConstruct(UIObject construct) {
    //if (construct.equals(uiConstruct)) return;
    uiConstruct = construct;
    getModel().setObject(construct);
  }
  public void setUIConstructOnly(UIObject construct) {
    uiConstruct = construct;
    getModel().setObject(construct);
  }
  public UIObject getUIConstruct() {
    return uiConstruct;
  }

  public void loadFromObject() {
    displayModelData();
  }

  //----------------------------A Renderer for Actions--------------------

  private class ActionListRenderer extends IXRenderer {
    public ActionListRenderer(boolean b) {super(b);}

    protected void setInfo(Object object){
      if (object == null) setText("");
      else {
	String name;
	LList pattern;
	if (Refinement.class.isAssignableFrom(object.getClass())) {
	  Refinement r = (Refinement)object;
	  String text = makeText(r.getName(), r.getPattern());
	  setText(text);
	}
	else if (UIRefinement.class.isAssignableFrom(object.getClass())) {
	  UIRefinement r = (UIRefinement)object;
	  String text = makeText(r.getName(), r.getPattern());
	  setText(text);
	}
	else if (NodeSpec.class.isAssignableFrom(object.getClass())) {
	  NodeSpec n = (NodeSpec)object;
	  String text = makeText(n.getId().toString(), n.getPattern());
	  setText(text);
	}
	else {
	  Debug.noteln("Can only render refinements in global form panels");
	}
      }
    }

    private String makeText(String name, List pattern) {
      String text = "";
      for (Iterator i = pattern.iterator(); i.hasNext();) {
	text = text + i.next().toString() + " ";
      }
      text = name + " (" + text + ")";
      return text;
    }

  }


  private class DescriptionsRenderer extends IXRenderer {
    public DescriptionsRenderer(boolean b) {super(b);}

    protected void setInfo(Object object){
      if (object == null) setText("");
      else {
	if (Annotations.class.isInstance(object)) {
	  //Object c = ((Annotations)object).getAnnotation((Object)"comments");
	  //setText(c.toString());
	}
      }
    }

  }

  


}
