/****************************************************************************
 * A grammar editor/viewer panel 
 *
 * @author Jussi Stader
 * @version 2.0
 * Copyright: (c) 2001, AIAI, University of Edinburgh
 *
 *****************************************************************************
 */
package ix.iview;

import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
//import java.awt.*;
import java.awt.GridLayout;
import java.awt.event.*;

import ix.*;
import ix.util.Debug;
import ix.util.Util;
import ix.util.lisp.*;
import ix.icore.*;
import ix.icore.domain.*;
import ix.icore.domain.event.*;
import ix.iface.ui.*;
import ix.iface.ui.util.*;
import ix.iface.domain.*;
import ix.iview.*;
import ix.iview.domain.*;
import ix.iview.domain.event.*;

/****************************************************************************
 * A grammar editor/viewer panel
 *
 * Waiting to be documented and implemented properly. <p>
 * Example code for using the GrammarEditorPanel:
 *<PRE><code>
 *    ...
 *    ...
 *</code></PRE>
 *****************************************************************************
 */
public class GrammarEditorPanel extends AConstructFramePanel
implements ActionListener, UIDomainListener {
  private ListEditorPanel issuePatternList;
  private ListEditorPanel nodePatternList;
  private ListEditorPanel constraintPatternList;

  private HashSet issuePatterns = new HashSet();
  private HashSet nodePatterns = new HashSet();
  private HashSet constraintPatterns = new HashSet();

  public GrammarEditorPanel(DomainEditor frame) {
    super(frame);
    updatePatterns();
  }

  protected JPanel setupMainPanel() {
    //scroll panes have ONE child, so use mainPanel for everything
    //mainPanel = new JPanel(new BorderLayout());
    mainPanel = new JPanel(new GridLayout(3, 1));

    scrollPane.getViewport().add(mainPanel); //mainPanel
    
    MouseListener ml = new MouseAdapter() {};

    //JScrollPane jsp = new JScrollPane();
    //jsp.getViewport().add(list);
    //nodePatternList = 
    //  new ListEditorPanel(ml,"Node Patterns",jsp,buttons);
    issuePatternList = makeList(ml, "Issue Patterns");
    nodePatternList = makeList(ml, "Node Patterns");
    constraintPatternList = makeList(ml, "Constraint Patterns");
    //mainPanel.add(patternList, BorderLayout.CENTER);
    return mainPanel;
  }

  public boolean beforeSwitching() {
    return true;
  }
     
  private ListEditorPanel makeList(MouseListener ml, String label) {
    String[] buttons = {"Add","Delete","Clear"};
    JList list = new JList();
    list.setVisibleRowCount(5);
    list.setCellRenderer(new ListRenderer(false));
    ListEditorPanel lep = new ListEditorPanel(ml, label, list, buttons);
    lep.setEnabled(false);
    mainPanel.add(lep);
    return lep;
  }


  /** 
   * Make a new construct of the panel's type and get ready to edit it.
   */
  public void newConstruct() {
    setConstruct((UIObject)null);
  }
  public UIObject makeNewConstruct() {
    return null;
  }
  /** 
   * Save a construct of the panel's type into the draft domain.
   */
  public void saveConstruct() {
    UIUtil.notImplemented(this,"Saving grammars");
  }
  /** 
   * Notes the content of the editing panel into the current(new?) UIObject.
   */
  protected UIObject noteConstruct() {
    return null;
  }
  /** 
   * Put a given construct of the panel's type into the panel ready for editing
   */
  public void setConstruct(UIObject construct) {
    if (construct == null) {
      issuePatternList.removeData();
      nodePatternList.removeData();
      constraintPatternList.removeData();
    }
    else UIUtil.notImplemented(this,"Setting grammars");
  }
  /** 
   *  Check a construct of the panel's type for errors and inconsistencies
   */
  public boolean checkConstruct() {
    UIUtil.notImplemented(this,"Checking grammars");
    return true;
  }
  /** 
   * Get a full list of the constructs of the panel's type in the domain.
   */
  public List getAllConstructs() {
    return new LListCollector();
  }
  public Named getNamedConstruct(String name) {
    return null;
  }

  public void constructAdded(UIDomainEvent se) {
    updatePatterns();
  }
  public void constructAdded(RefinementEvent se) {
    updatePatterns();
  }
  public void constructEdited(UIDomainEvent se) {
    updatePatterns();
  }
  public void constructRemoved(UIDomainEvent se) {
    updatePatterns();
  }
  public void domainCleared(UIDomainEvent se) {
    updatePatterns();
  }
  public void domainEdited(UIDomainEvent se) {
  }
  public void domainSet(UIDomainEvent se) {
    updatePatterns();
  }

  private void updatePatterns() {
    issuePatterns.clear();
    nodePatterns.clear();
    constraintPatterns.clear();
    if (parent.getUIDomain() != null) {
      List refs = parent.getUIDomain().getAllRefinements();
      for (Iterator i = refs.iterator(); i.hasNext(); ) 
	addAllPatterns(i.next());
    }
    updateListPatterns(issuePatternList, issuePatterns);
    updateListPatterns(nodePatternList, nodePatterns);
    updateListPatterns(constraintPatternList, constraintPatterns);
  }

  private void addAllPatterns(Object o) {
    if (o == null) return;

    Iterator i = null;
    List rn = null;
    List ri = null;
    List rc = null;

    Object pattern = null;
    if (Refinement.class.isInstance(o)) {
      Refinement r = (Refinement)o;
      nodePatterns.add(r.getPattern());
      rn = r.getNodes();
      ri = r.getIssues();
      rc = r.getConstraints();
    }
    else if (UIRefinement.class.isInstance(o)) {
      UIRefinement r = (UIRefinement)o;
      nodePatterns.add(r.getPattern());
      rn = r.getNodes();
      ri = r.getIssues();
      rc = r.getConstraints();
    }
    else {
      Debug.noteln("GEP: Cannot get patterns from class ", 
		   o.getClass().getName());
      return;
    }

    if (rn != null)  
      nodePatterns = getNodePatterns(rn.iterator(), nodePatterns);
    if (ri != null)  
      issuePatterns = getIssuePatterns(ri.iterator(), issuePatterns);
    if (rc != null)  
      constraintPatterns = 
	getConstraintPatterns(rc.iterator(), constraintPatterns);
  }


  private void updateListPatterns(ListEditorPanel list, HashSet patterns) {
    LinkedList pList = new LinkedList(patterns);
    Collections.sort(pList, new UIUtil.ToStringComparator());
    list.setData(pList.toArray());
  }

  public HashSet getAllPatterns() {
    if (parent.getUIDomain() == null) return null;
    else {
      List refs = parent.getUIDomain().getAllRefinements();
      Iterator i = refs.iterator();
      HashSet patterns = new HashSet();
      while (i.hasNext()) 
	patterns.addAll(getAllPatterns(i.next()));
      return patterns;
    }
  }
  public HashSet getAllPatterns(Object o) {
    HashSet patterns = new HashSet();
    if (o == null) return patterns;
    Iterator i = null;
    List rn = null;
    List ri = null;

    Object pattern = null;
    if (Refinement.class.isInstance(o)) {
      Refinement r = (Refinement)o;
      rn = r.getNodes();
      if (rn != null)  patterns = getNodePatterns(rn.iterator(), patterns);
      ri = r.getIssues();
      if (ri != null)  patterns = getIssuePatterns(ri.iterator(), patterns);
      patterns.add(r.getPattern());
    }
    else if (UIRefinement.class.isInstance(o)) {
      UIRefinement r = (UIRefinement)o;
      rn = r.getNodes();
      if (rn != null)  patterns = getNodePatterns(rn.iterator(), patterns);
      ri = r.getIssues();
      if (ri != null)  patterns = getIssuePatterns(ri.iterator(), patterns);
      patterns.add(r.getPattern());
    }
    else {
      Debug.noteln("GEP: Cannot get patterns from class ", 
		   o.getClass().getName());
      return null;
    }

    return patterns;
  }

  public HashSet getNodePatterns(Iterator i, HashSet patterns) {
    if (i != null) 
      while (i.hasNext()) 
	patterns.add(((NodeSpec)i.next()).getPattern());  
    return patterns;
  }
  public HashSet getIssuePatterns(Iterator i, HashSet patterns) {
    if (i != null) 
      while (i.hasNext()) 
	patterns.add(((TaskItem)i.next()).getPattern());  
    return patterns;
  }
  public HashSet getConstraintPatterns(Iterator i, HashSet patterns) {
    //Debug.noteln("Cannot collect constraint patterns yet");
    return patterns;
  }


  public boolean frameActionPerformed(ActionEvent event){
    String command = event.getActionCommand();
    if (command == "newConstruct") newConstruct();
    else if (command == "editConstruct") {
      //***dummy
    }
    else if (command == "modify") saveConstruct();
    else if (command == "check") checkConstruct();
    else return false;
    return true;
  }


  /**
   * Wakes up the ActionListener with a user action.
   * This is called when a KeyStroke happens in which the ActionListener
   * registered its interest.
   */
  public void actionPerformed(ActionEvent ae){
    String command = ae.getActionCommand();
    Debug.noteln("Grammar editor: Got action command", command);
  }

}

/*Issues***************************************************************
 *
 * implement this?
 *
/*Todos***************************************************************
 *
 *****************************************************************************
 */
