/****************************************************************************
 * Utilities for IView things
 *
 * @author Jussi Stader
 * @version 2.0
 * Copyright: (c) 2001, AIAI, University of Edinburgh
 *
 *****************************************************************************
 */

package ix.iview;

import java.util.*;

import ix.util.*;
import ix.util.Debug;
import ix.util.Util;
import ix.icore.*;
import ix.icore.domain.*;

/****************************************************************************
 * Utilities for IView things
 *****************************************************************************
 */
public abstract class IViewUtil {

  public static Collection sortNamedCollection(Collection c) {
    Map sortMap = new TreeMap();
    Collection empties = new ArrayList(); //collect empty names seperately
    for (Iterator i = c.iterator(); i.hasNext(); ) {
      Named n = (Named)i.next();
      if (n != null) { //ignore null elements
	String name = n.getName();
	if ((name == null) || (name.equals(""))) empties.add(n);
	else sortMap.put(name, n);
      }
    }
    Collection sorted = new ArrayList(sortMap.values()); 
    sorted.addAll(empties);
    return sorted;
  }

  public static List sortNodeCollection(Collection c) {
    Map sortMap = new TreeMap();
    for (Iterator i = c.iterator(); i.hasNext(); ) {
      NodeSpec n = (NodeSpec)i.next();
      if (n != null) { //ignore null elements
	Name id = n.getId();
	sortMap.put(id.toString(), n);
      }
    }
    List sorted = new ArrayList(sortMap.values()); 
    return sorted;
  }  

  public static List sortTICollection(Collection c) {
    Map sortMap = new TreeMap();
    for (Iterator i = c.iterator(); i.hasNext(); ) {
      TaskItem ti = (TaskItem)i.next();
      if (ti != null) { //ignore null elements
	Name id = ti.getId();
	sortMap.put(id.toString(), ti);
      }
    }
    List sorted = new ArrayList(sortMap.values()); 
    return sorted;
  }  

  public static String printNodes(Collection nodes) {
    String printed = "";
    for (Iterator i = nodes.iterator(); i.hasNext(); ) 
      printed = printed + ", " + printNode((NodeSpec)i.next());
    return printed;
  }
  public static String printOrderings(Collection orderings) {
    String printed = "";
    for (Iterator i = orderings.iterator(); i.hasNext(); ) 
      printed = printed + ", " + printOrdering((Ordering)i.next());
    return printed;
  }

  public static String printNode(NodeSpec node) {
    return node.getPattern().toString();
  }
  public static String printOrdering(Ordering ordering) {
    if (ordering == null) return "";

    NodeEndRef fromE = ordering.getFrom();
    NodeEndRef toE = ordering.getTo();
    String f = "";
    String ft = "";
    String t = "";
    String tt = "";
    if ((fromE != null) && (fromE.getNode() != null))
      f = fromE.getNode().toString();
    if ((fromE != null) && (fromE.getEnd() != null))
      ft = fromE.getEnd().toString();
    if ((toE != null) && (toE.getNode() != null))
      t = toE.getNode().toString();
    if ((toE != null) && (toE.getEnd() != null))
      tt = toE.getEnd().toString();
    return f + "-" + ft + " -> " + t + "-" + tt;
  }

}
