/****************************************************************************
 * A simple text-input editor for entering specifications, e.g. for nodes
 * and issues.
 *
 * @author Jussi Stader
 * @version 2.0
 * Copyright: (c) 2001, AIAI, University of Edinburgh
 *
 *****************************************************************************
 */

package ix.iview;

import javax.swing.*;       
import java.util.*;
import java.awt.Color;
import java.awt.Component;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import ix.*;
import ix.util.*;
import ix.util.Debug;
import ix.util.lisp.LList;
import ix.util.lisp.Lisp;
import ix.icore.domain.*;
import ix.icore.*;
import ix.iview.domain.*;
import ix.iface.ui.*;
import ix.iface.ui.util.*;

/****************************************************************************
 * A simple text-input editor for entering specifications, e.g. for nodes
 * and issues. It is like a JOptions dialogue, but it has an "Add More" 
 * button.
 *
 * The editor is implemented as a frame using JTextField objects
 * and JButtons.  <p>
 *
 * The editor can be created with no arguments or with a given refinement
 * which will be set as the current refinement for which the specifications
 * are to be defined. Subsequently, the refinement in the editor can be set
 * using the setRefinement method.
 *****************************************************************************
 */
public class SpecEditor extends ConstraintEditor
{
  
  public String type;
  public String message;
  
  /**
   * Creates an editor for specifying time point relations.
   * The editor has components for selecting two time points of two
   * sub-activities, and two buttons (Ok, Cancel)
   */
  public SpecEditor(String type) {
    this(type, "Please enter the specification.");
  }
    
  /**
   * Creates an editor for specifying time point relations.
   * The editor has components for selecting two time points of two
   * sub-activities, and two buttons (Ok, Cancel)
   */
  public SpecEditor(String type, String message) {
    super("Specification Editor", "");
    this.message = message;
    this.type = type;
    this.setSize(405, 120);
  }
    
  
  /**
   * Makes all items on the relations panel - just a text field.
   */
  protected Component makeRels() {
    JPanel relsPanel = new IXEdPanel(null, message, new JTextField());
    return relsPanel;
  }

  public void setMessage(String message) {
    this.message = message;
    ((IXEdPanel)relsBox).setLabel(message);
  }

  public void setType(String type) {
    this.type = type;
  }
  public void setText(String text) {
    ((IXEdPanel)relsBox).setTextData(text);
  }


  public void start() {
    setVisible(true);
    initFromObject();
  }

  /**
   * Nothing to do to initialise the editor from an object.
   */
  protected void initFromObject() {
    setText("");
    ((IXEdPanel)relsBox).getItemComponent().requestFocus();
  }
  /**
   * Nothing to do to initialise the editor from an object.
   */
  protected void setFromObject(Object original) {
    LList pattern = Lisp.NIL;
    if (Issue.class.isInstance(original)) 
      pattern = ((Issue)original).getPattern();
    else if (NodeSpec.class.isInstance(original)) 
      pattern = ((NodeSpec)original).getPattern();
    
    setText(UIUtil.listToDisplay(pattern));
    ((IXEdPanel)relsBox).getItemComponent().requestFocus();
  }


  /**
   * Reads the selected relation and let interested parties know.
   * Checks that selections have been made.
   */
  protected Object collectConstraint() {
    String spec = ((IXEdPanel)relsBox).getTextData();
    if (spec == "") return null;
    else {
      LList pSpec = Lisp.elementsFromString(spec);
      if (isEditing()) {
	//update the original and return null.
	if (Issue.class.isInstance(original)) 
	  ((Issue)original).setPattern(pSpec);
	else if (NodeSpec.class.isInstance(original)) 
	  ((NodeSpec)original).setPattern(pSpec);
	return null;
      }
      else return new UISpec(type, pSpec);
    }
  }

}

