/****************************************************************************
 * A graph with layout and view information.
 *
 * @author Jussi Stader
 * @version 2.0
 * Copyright: (c) 2002, AIAI, University of Edinburgh
 *
 *****************************************************************************
 */

package ix.iview.igraph;


import java.util.*;
import java.awt.*;
import javax.swing.*;
import java.io.*;
import lt.monarch.graph.*;
//import lt.monarch.graph.plugins.*;
import lt.monarch.graph.view.*;

import ix.*;
import ix.util.Debug;
import ix.util.Util;
import ix.util.lisp.*;
import ix.iface.ui.*;
import ix.icore.domain.*;
import ix.iview.domain.*;

/**
 * A graph with layout and view information.
 */
public class IGraph extends Object implements Named {
  public Class nodeViewType = NodeView.class;
  public Class nodeType = Object.class;
  //public Class baseObjectClass;
  public String name;
  //public Object baseObject;
  public LListCollector nodes = new LListCollector();

  /**
   *Creates a new graph object with the given name and information from the 
   *given graph view. Notes each model node with its location.
   *@param theName the graph's name
   *@param graph the GraphView object that holds the MGraph information
   */
  public IGraph(String theName, GraphView graphView) {
    super();
    name = theName;
    refreshGraph(graphView);
  }


  /**
   *Refreshes the IGraph from the information in the graphView.
   *@param graph the GraphView object that holds the MGraph information
   */
  public void refreshGraph(GraphView graphView) {
    //Debug.noteln("IGraph: refreshing graph");
    Iterator nodeViews = graphView.nodeLayer.getViews();
    nodes.clear();
    NodeView nodeView = null;
    NodeSpec node = null;
    Point position;
    while (nodeViews.hasNext()) {
      nodeView = (NodeView)nodeViews.next();
      position = nodeView.getPosition();
      node = (NodeSpec)nodeView.getModelNode();
      //Debug.noteln(" put node/pos " + node.toString() + position.toString());
      nodes.add(new GraphNodeComponent(position, node));
    }
    if (nodeView != null) nodeViewType = nodeView.getClass();
    if (node != null) nodeType = node.getClass();
  }

  public Point getPosition(Object node) {
    GraphNodeComponent gnc = findNode(node);
    if (gnc == null) {
      Debug.noteln("IGraph: getPosition cannot find node " + node.toString());
      return null;
    }
    else return gnc.position;
  }

  protected GraphNodeComponent findNode(Object node) {
    Enumeration e = nodes.elements();
    while (e.hasMoreElements()) {
      GraphNodeComponent n = (GraphNodeComponent)e.nextElement();
      //Debug.noteln("IGraph: findNode checking node " + n.toString());
      if (UIRefinement.isRef(n.baseNode, node)) return n;
    }
    return null;
  }

  public String getName() {
    return name;
  }
  public Object clone() throws CloneNotSupportedException {
    return super.clone();
  }


  private class GraphNodeComponent extends Object {
    public Point position;
    public double width;
    public double height;
    public NodeSpec baseNode;

    public GraphNodeComponent(Point p, NodeSpec base) {
      super();
      position = p;
      baseNode = base;
    }
    public GraphNodeComponent(Point p, double w, double h, NodeSpec base) {
      this(p, base);
      width = w;
      height = h;
    }
  }

}
