/* Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Updated: Wed May 29 22:31:41 2002 by Jeff Dalton
 * Copyright: (c) 2002, AIAI, University of Edinburgh
 */

package ix.util;

import javax.swing.*;
import java.util.*;

/**
 * A thread that catches and reports errors or exceptions.
 */
public class CatchingThread extends Thread {

    Runnable innerRunnable;

    public CatchingThread() {
    }

    public CatchingThread(Runnable innerRunnable) {
	this.innerRunnable = innerRunnable;
    }

    /**
     * Calls innerRun() inside a catch that calls the handleException
     * method on any Throwable caught.
     */
    public void run() {
	try {
	    innerRun();
	}
	catch (Throwable except) {
	    Debug.noteException(except);
	    handleException(except);
	}
    }

    /**
     * Calls the run() method of the innerRunnable passed to the
     * constructor, if innerRunnable is non-null, otherwise does
     * nothing.
     */
    public void innerRun() {
	if (innerRunnable != null)
	    innerRunnable.run();
	// Else nothing to do unless this method is redefined
	// by a subclass.
    }

    /**
     * Uses SwingUtilities.invokeLater to call the reportException
     * method in the event-handling thread.
     */
    protected void handleException(final Throwable except) {
        SwingUtilities.invokeLater(new Runnable() {
	    public void run() {
		reportException(except);
	    }
	});
    }

    /**
     * Puts up a description of the exception in a message dialog.
     */
    protected void reportException(Throwable except) {
	String descr = ix.iface.util.CatchingActionListener // /\/
	    .description(except);
	JOptionPane.showMessageDialog(null,
	    new Object[] {descr},
	    "Exception",
	    JOptionPane.ERROR_MESSAGE);
    }

}
