/* Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Updated: Tue Jun 11 17:15:50 2002 by Jeff Dalton
 * Copyright: (c) 2002, AIAI, University of Edinburgh
 */

package ix.util.xml;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.*;

import ix.icore.IXAgent;

import ix.util.*;
import ix.util.lisp.*;

/** 
 * An XML tree editor as a standalone application main program.
 *
 * @see XMLTreeEditFrame
 * @see XMLTreeEditPanel
 */

public class XMLTreeEditor extends IXAgent {

    protected XMLTreeEditFrame frame;

    public XMLTreeEditor() {
	displayName = "I-X XML Tree Editor";

	// Change Icons used for "one-touch" expand / collapse operation.
	// It appears that we have to do this globally.  /\/
	UIManager.put("Tree.expandedIcon",
		      Util.resourceImageIcon("ix-symbol-fold.gif"));
	UIManager.put("Tree.collapsedIcon",
		      Util.resourceImageIcon("ix-symbol-unfold.gif"));

    }

    /**
     * Main program.
     */
    public static void main(String[] argv) {

	Util.printGreeting("XML Tree Editor");

	new XMLTreeEditor().mainStartup(argv);

    }

    /**
     * Command-line argument processing.  There are at present no
     * command-line arguments beyond those used by all IXAgents.
     *
     * @see ix.icore.IXAgent#processCommandLineArguments()
     */
    protected void processCommandLineArguments() {
	super.processCommandLineArguments();
    }

    /**
     * Completes setup and initialization.
     */
    public void startup() {
	frame = new XMLTreeEditFrame(this, displayName);

	// Make sure a "close" doesn't just leave it running in the
	// background...
	frame.setDefaultCloseOperation(frame.DO_NOTHING_ON_CLOSE);
	frame.addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {
		Debug.noteln("Main frame windowClosing event");
		frame.exitIfUserConfirms();
	    }
	});

	frame.setVisible(true);

    }

}
